@@ -1,9 +1,12 @@
 const ALLOW_EMPTY_CLASS = UInt32(0x00000001)
 const ALT_BSUX = UInt32(0x00000002)
 const ALT_CIRCUMFLEX = UInt32(0x00200000)
+const ALT_EXTENDED_CLASS = UInt32(0x08000000)
 const ALT_VERBNAMES = UInt32(0x00400000)
 const ANCHORED = UInt32(0x80000000)
 const AUTO_CALLOUT = UInt32(0x00000004)
+const AUTO_POSSESS = UInt32(64)
+const AUTO_POSSESS_OFF = UInt32(65)
 const BSR_ANYCRLF = UInt32(2)
 const BSR_UNICODE = UInt32(1)
 const CALLOUT_BACKTRACK = UInt32(0x00000002)
@@ -40,6 +43,8 @@ const DFA_SHORTEST = UInt32(0x00000080)
 const DISABLE_RECURSELOOP_CHECK = UInt32(0x00040000)
 const DOLLAR_ENDONLY = UInt32(0x00000010)
 const DOTALL = UInt32(0x00000020)
+const DOTSTAR_ANCHOR = UInt32(66)
+const DOTSTAR_ANCHOR_OFF = UInt32(67)
 const DUPNAMES = UInt32(0x00000040)
 const ENDANCHORED = UInt32(0x20000000)
 const ERROR_ALPHA_ASSERTION_UNKNOWN = Cint(195)
@@ -70,6 +75,7 @@ const ERROR_BAD_RELATIVE_REFERENCE = Cint(129)
 const ERROR_BAD_SUBPATTERN_REFERENCE = Cint(115)
 const ERROR_CALLOUT = Cint((-37))
 const ERROR_CALLOUT_BAD_STRING_DELIMITER = Cint(182)
+const ERROR_CALLOUT_CALLER_DISABLED = Cint(203)
 const ERROR_CALLOUT_NO_STRING_DELIMITER = Cint(181)
 const ERROR_CALLOUT_NUMBER_TOO_BIG = Cint(138)
 const ERROR_CALLOUT_STRING_TOO_LONG = Cint(172)
@@ -77,7 +83,6 @@ const ERROR_CLASS_INVALID_RANGE = Cint(150)
 const ERROR_CLASS_RANGE_ORDER = Cint(108)
 const ERROR_CODE_POINT_TOO_BIG = Cint(134)
 const ERROR_CONDITION_ASSERTION_EXPECTED = Cint(128)
-const ERROR_CONDITION_ATOMIC_ASSERTION_EXPECTED = Cint(198)
 const ERROR_CONVERT_SYNTAX = Cint((-64))
 const ERROR_DEFINE_TOO_MANY_BRANCHES = Cint(154)
 const ERROR_DEPTHLIMIT = Cint((-53))
@@ -89,10 +94,18 @@ const ERROR_DFA_UINVALID_UTF = Cint((-66))
 const ERROR_DFA_UITEM = Cint((-42))
 const ERROR_DFA_WSSIZE = Cint((-43))
 const ERROR_DUPLICATE_SUBPATTERN_NAME = Cint(143)
+const ERROR_ECLASS_EXPECTED_OPERAND = Cint(210)
+const ERROR_ECLASS_HINT_SQUARE_BRACKET = Cint(212)
+const ERROR_ECLASS_INVALID_OPERATOR = Cint(208)
+const ERROR_ECLASS_MIXED_OPERATORS = Cint(211)
+const ERROR_ECLASS_NEST_TOO_DEEP = Cint(207)
+const ERROR_ECLASS_UNEXPECTED_OPERATOR = Cint(209)
 const ERROR_END_BACKSLASH = Cint(101)
 const ERROR_END_BACKSLASH_C = Cint(102)
 const ERROR_ESCAPE_INVALID_IN_CLASS = Cint(107)
 const ERROR_ESCAPE_INVALID_IN_VERB = Cint(140)
+const ERROR_EXTRA_CASING_INCOMPATIBLE = Cint(206)
+const ERROR_EXTRA_CASING_REQUIRES_UNICODE = Cint(204)
 const ERROR_HEAPLIMIT = Cint((-63))
 const ERROR_HEAP_FAILED = Cint(121)
 const ERROR_INTERNAL = Cint((-44))
@@ -116,6 +129,7 @@ const ERROR_INVALID_OCTAL = Cint(164)
 const ERROR_INVALID_SUBPATTERN_NAME = Cint(144)
 const ERROR_JIT_BADOPTION = Cint((-45))
 const ERROR_JIT_STACKLIMIT = Cint((-46))
+const ERROR_JIT_UNSUPPORTED = Cint((-68))
 const ERROR_LOOKBEHIND_INVALID_BACKSLASH_C = Cint(136)
 const ERROR_LOOKBEHIND_NOT_FIXED_LENGTH = Cint(125)
 const ERROR_LOOKBEHIND_TOO_COMPLICATED = Cint(135)
@@ -123,11 +137,13 @@ const ERROR_LOOKBEHIND_TOO_LONG = Cint(187)
 const ERROR_MALFORMED_UNICODE_PROPERTY = Cint(146)
 const ERROR_MARK_MISSING_ARGUMENT = Cint(166)
 const ERROR_MATCHLIMIT = Cint((-47))
+const ERROR_MAX_VAR_LOOKBEHIND_EXCEEDED = Cint(200)
 const ERROR_MISSING_CALLOUT_CLOSING = Cint(139)
 const ERROR_MISSING_CLOSING_PARENTHESIS = Cint(114)
 const ERROR_MISSING_COMMENT_CLOSING = Cint(118)
 const ERROR_MISSING_CONDITION_CLOSING = Cint(124)
 const ERROR_MISSING_NAME_TERMINATOR = Cint(142)
+const ERROR_MISSING_OCTAL_DIGIT = Cint(198)
 const ERROR_MISSING_OCTAL_OR_HEX_DIGITS = Cint(178)
 const ERROR_MISSING_SQUARE_BRACKET = Cint(106)
 const ERROR_MIXEDTABLES = Cint((-30))
@@ -139,13 +155,19 @@ const ERROR_NO_SURROGATES_IN_UTF16 = Cint(191)
 const ERROR_NULL = Cint((-51))
 const ERROR_NULL_PATTERN = Cint(116)
 const ERROR_OCTAL_BYTE_TOO_BIG = Cint(151)
+const ERROR_OVERSIZE_PYTHON_OCTAL = Cint(202)
 const ERROR_PARENS_QUERY_R_MISSING_CLOSING = Cint(158)
 const ERROR_PARENTHESES_NEST_TOO_DEEP = Cint(119)
 const ERROR_PARENTHESES_STACK_CHECK = Cint(133)
 const ERROR_PARTIAL = Cint((-2))
+const ERROR_PATTERN_COMPILED_SIZE_TOO_BIG = Cint(201)
 const ERROR_PATTERN_STRING_TOO_LONG = Cint(188)
 const ERROR_PATTERN_TOO_COMPLICATED = Cint(186)
 const ERROR_PATTERN_TOO_LARGE = Cint(120)
+const ERROR_PERL_ECLASS_EMPTY_EXPR = Cint(214)
+const ERROR_PERL_ECLASS_MISSING_CLOSE = Cint(215)
+const ERROR_PERL_ECLASS_UNEXPECTED_CHAR = Cint(216)
+const ERROR_PERL_ECLASS_UNEXPECTED_EXPR = Cint(213)
 const ERROR_POSIX_CLASS_NOT_IN_CLASS = Cint(112)
 const ERROR_POSIX_NO_SUPPORT_COLLATING = Cint(113)
 const ERROR_QUANTIFIER_INVALID = Cint(109)
@@ -154,6 +176,7 @@ const ERROR_QUANTIFIER_TOO_BIG = Cint(105)
 const ERROR_QUERY_BARJX_NEST_TOO_DEEP = Cint(184)
 const ERROR_RECURSELOOP = Cint((-52))
 const ERROR_RECURSIONLIMIT = Cint((-53))
+const ERROR_REPLACECASE = Cint((-69))
 const ERROR_REPMISSINGBRACE = Cint((-58))
 const ERROR_SCRIPT_RUN_NOT_AVAILABLE = Cint(196)
 const ERROR_SUBPATTERN_NAMES_MISMATCH = Cint(165)
@@ -161,10 +184,12 @@ const ERROR_SUBPATTERN_NAME_EXPECTED = Cint(162)
 const ERROR_SUBPATTERN_NAME_TOO_LONG = Cint(148)
 const ERROR_SUBPATTERN_NUMBER_TOO_BIG = Cint(161)
 const ERROR_SUPPORTED_ONLY_IN_UNICODE = Cint(193)
+const ERROR_TOOLARGEREPLACE = Cint((-70))
 const ERROR_TOOMANYREPLACE = Cint((-61))
 const ERROR_TOO_MANY_CAPTURES = Cint(197)
 const ERROR_TOO_MANY_CONDITION_BRANCHES = Cint(127)
 const ERROR_TOO_MANY_NAMED_SUBPATTERNS = Cint(149)
+const ERROR_TURKISH_CASING_REQUIRES_UTF = Cint(205)
 const ERROR_UCP_IS_DISABLED = Cint(175)
 const ERROR_UNAVAILABLE = Cint((-54))
 const ERROR_UNICODE_DISALLOWED_CODE_POINT = Cint(173)
@@ -224,6 +249,10 @@ const EXTRA_CASELESS_RESTRICT = UInt32(0x00000080)
 const EXTRA_ESCAPED_CR_IS_LF = UInt32(0x00000010)
 const EXTRA_MATCH_LINE = UInt32(0x00000008)
 const EXTRA_MATCH_WORD = UInt32(0x00000004)
+const EXTRA_NEVER_CALLOUT = UInt32(0x00008000)
+const EXTRA_NO_BS0 = UInt32(0x00004000)
+const EXTRA_PYTHON_OCTAL = UInt32(0x00002000)
+const EXTRA_TURKISH_CASING = UInt32(0x00010000)
 const FIRSTLINE = UInt32(0x00000100)
 const INFO_ALLOPTIONS = UInt32(0)
 const INFO_ARGOPTIONS = UInt32(1)
@@ -257,11 +286,12 @@ const JIT_COMPLETE = UInt32(0x00000001)
 const JIT_INVALID_UTF = UInt32(0x00000100)
 const JIT_PARTIAL_HARD = UInt32(0x00000004)
 const JIT_PARTIAL_SOFT = UInt32(0x00000002)
+const JIT_TEST_ALLOC = UInt32(0x00000200)
 const LITERAL = UInt32(0x02000000)
 const MAJOR = UInt32(10)
 const MATCH_INVALID_UTF = UInt32(0x04000000)
 const MATCH_UNSET_BACKREF = UInt32(0x00000200)
-const MINOR = UInt32(43)
+const MINOR = UInt32(46)
 const MULTILINE = UInt32(0x00000400)
 const NEVER_BACKSLASH_C = UInt32(0x00100000)
 const NEVER_UCP = UInt32(0x00000800)
@@ -282,8 +312,15 @@ const NO_DOTSTAR_ANCHOR = UInt32(0x00008000)
 const NO_JIT = UInt32(0x00002000)
 const NO_START_OPTIMIZE = UInt32(0x00010000)
 const NO_UTF_CHECK = UInt32(0x40000000)
+const OPTIMIZATION_FULL = UInt32(1)
+const OPTIMIZATION_NONE = UInt32(0)
 const PARTIAL_HARD = UInt32(0x00000020)
 const PARTIAL_SOFT = UInt32(0x00000010)
+const START_OPTIMIZE = UInt32(68)
+const START_OPTIMIZE_OFF = UInt32(69)
+const SUBSTITUTE_CASE_LOWER = UInt32(1)
+const SUBSTITUTE_CASE_TITLE_FIRST = UInt32(3)
+const SUBSTITUTE_CASE_UPPER = UInt32(2)
 const SUBSTITUTE_EXTENDED = UInt32(0x00000200)
 const SUBSTITUTE_GLOBAL = UInt32(0x00000100)
 const SUBSTITUTE_LITERAL = UInt32(0x00008000)
